//----------------------------------------------------------------------
// File:			ALTSbasicStat.h
// Programmer:		David Mount
// Description:		LTS definitions for basic stats
// Last modified:	06/17/2015 (Version 1.0)
//----------------------------------------------------------------------
// Copyright (c) 2015 University of Maryland and David Mount.  All
// Rights Reserved.
// 
// This software and related documentation is part of the Approximate
// Least-Trimmed Squares (ALTS) software.  This software is provided
// under the provisions of the GNU Public License (GPL). See the file
// ReadMe.txt for further information.
// 
// The University of Maryland (U.M.) and the authors make no
// representations about the suitability or fitness of this software for
// any purpose.  It is provided "as is" without express or implied
// warranty.
//----------------------------------------------------------------------
// History:
//	Revision 1.0  06/17/2015
//		Initial release
//----------------------------------------------------------------------

#ifndef ALTS_BASIC_STAT
#define ALTS_BASIC_STAT

#include "ALTS.h"						// basic ALTS definitions

//----------------------------------------------------------------------
//	Additions to alts namespace
//----------------------------------------------------------------------

namespace alts {

	class BasicStat;					// basic statistics
}

//----------------------------------------------------------------------
//	BasicStat
//	Maintains a basic Scalar statistic, with methods to extract the
//	mean and variance of the statistic.
//
//		reset()			Reset to no samples
//		+= x			Include sample x
//		-= x			Remove sample x
//		nSamples()		Return number of samples
//		mean()			Return mean of samples
//		variance()		Return variance
//----------------------------------------------------------------------

class alts::BasicStat {
	int n;								// number of samples
	Scalar sum;							// sum
	Scalar sumSq;						// sum of squares
	Scalar meanVal;						// mean value
	Scalar varVal;						// variance
	bool dirty;							// need to recompute mean & var
private:								// private utilities
	void update();						// update mean and variance
public :
	void reset()						// reset everything
	{
		n = 0;
		sum = sumSq = 0;
		dirty = true;
	}

	BasicStat()							// constructor
	{ reset(); }

	void operator+=(Scalar x)			// add sample
	{  n++;  sum += x;	 sumSq += x*x;  dirty = true; }

	void operator-=(Scalar x)			// remove sample
	{  n--;  sum -= x;	 sumSq -= x*x;  dirty = true; }

	int	 nSamples() const				// number of samples
	{ return n; }

	Scalar mean();						// mean

	Scalar variance();					// variance

	Scalar stdDev();					// standard deviation
};

#endif
