//----------------------------------------------------------------------
// File:			ALTSutilities.h
// Programmer:		David Mount
// Description:		LTS utilities definitions
// Last modified:	06/17/2015 (Version 1.0)
//----------------------------------------------------------------------
// Copyright (c) 2015 University of Maryland and David Mount.  All
// Rights Reserved.
// 
// This software and related documentation is part of the Approximate
// Least-Trimmed Squares (ALTS) software.  This software is provided
// under the provisions of the GNU Public License (GPL). See the file
// ReadMe.txt for further information.
// 
// The University of Maryland (U.M.) and the authors make no
// representations about the suitability or fitness of this software for
// any purpose.  It is provided "as is" without express or implied
// warranty.
//----------------------------------------------------------------------
// History:
//	Revision 1.0  06/17/2015
//		Initial release
//----------------------------------------------------------------------

#ifndef ALTS_UTILITIES_H
#define ALTS_UTILITIES_H

#include "ALTS.h"						// basic ALTS definitions

//----------------------------------------------------------------------
//	namespace additions
//----------------------------------------------------------------------

namespace alts {
	class ScalarLTSResult;				// result of scalar LTS
	class ScalarLMSResult;				// result of scalar LMS
	class ScalarSet;					// set of scalars
	class ScalarRangeSet;				// set of scalar intervals

	void cropSamplesToRange(			// crop samples to range
		SlopeSet& 			samples, 	// the slope samples (modified)
		const SlopeRange& 	slopeRange); // slope range

	SlopeRange robustSlopeRange(		// robust bounding box for slopes
		int 				d,			// dimension
		SlopeSet& 			samples,	// the slope samples (modified)
		int					h);			// trimming value

	ScalarLTSResult intvLTS(			// interval LTS procedure
		const ScalarRangeSet&  intervals, // the intervals
		int					h);			// trimming value

	ScalarLTSResult intvLTSSpecial(		// interval LTS procedure
		const ScalarRangeSet&  intervals, // the intervals
		int					h);			// trimming value

	ScalarLTSResult scalarLTS(			// scalar LTS procedure
		const ScalarSet& 	pts, 		// the points
		int					h);			// trimming value
}

//----------------------------------------------------------------------
//	Scalar LTS result - result of LTS on scalar values
//----------------------------------------------------------------------

class alts::ScalarLTSResult {
public:
	Scalar	loc;						// best location
	Scalar	variance;					// the associated variance
public:
										// basic constructor
	ScalarLTSResult(Scalar ll = 0, Scalar var = 0)
		: loc(ll), variance(var) { }
};

//----------------------------------------------------------------------
//	ScalarSet - A set of scalars
//	This is basically a C++ STL vector of Scalars.
//----------------------------------------------------------------------

class alts::ScalarSet : public std::vector<Scalar> {
public:
	ScalarSet(int n = 0)				// construct from size
		: std::vector<Scalar>(n) { }
};

//----------------------------------------------------------------------
//	ScalarRangeSet - A set of scalar intervals
//	This is basically a C++ STL vector of ScalarRange.
//----------------------------------------------------------------------

class alts::ScalarRangeSet : public std::vector<ScalarRange> {
public:
	ScalarRangeSet(int n = 0)			// construct from size
		: std::vector<ScalarRange>(n) { }
};

#endif
